@@ -1,6 +1,6 @@
 /***
     This file is part of snapcast
-    Copyright (C) 2014-2024  Johannes Pohl
+    Copyright (C) 2014-2025  Johannes Pohl
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,7 +18,6 @@
 
 #pragma once
 
-class BrowseBonjour;
 
 // local headers
 #include "browse_mdns.hpp"
@@ -27,6 +26,7 @@ class BrowseBonjour;
 #include <dns_sd.h>
 
 
+/// Bonjour based mDNS browser
 class BrowseBonjour : public BrowsemDNS
 {
 public:
