@@ -1,6 +1,6 @@
 /***
     This file is part of snapcast
-    Copyright (C) 2014-2020  Johannes Pohl
+    Copyright (C) 2014-2023  Johannes Pohl
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,11 +16,21 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***/
 
+// prototype/interface header file
 #include "coreaudio_player.hpp"
+
+// local headers
+#include "common/aixlog.hpp"
+
+// 3rd party headers
 #include <CoreAudio/CoreAudio.h>
 
+namespace player
+{
+
 #define NUM_BUFFERS 2
 
+static constexpr auto LOG_TAG = "CoreAudioPlayer";
 
 // http://stackoverflow.com/questions/4863811/how-to-use-audioqueue-to-play-a-sound-for-mac-osx-in-c
 // https://gist.github.com/andormade/1360885
@@ -32,7 +42,8 @@ void callback(void* custom_data, AudioQueueRef queue, AudioQueueBufferRef buffer
 }
 
 
-CoreAudioPlayer::CoreAudioPlayer(const PcmDevice& pcmDevice, std::shared_ptr<Stream> stream) : Player(pcmDevice, stream), ms_(100), pubStream_(stream)
+CoreAudioPlayer::CoreAudioPlayer(boost::asio::io_context& io_context, const ClientSettings::Player& settings, std::shared_ptr<Stream> stream)
+    : Player(io_context, settings, stream), ms_(100), pubStream_(stream)
 {
 }
 
@@ -43,7 +54,7 @@ CoreAudioPlayer::~CoreAudioPlayer()
 
 
 /// TODO: experimental. No output device can be configured yet.
-std::vector<PcmDevice> CoreAudioPlayer::pcm_list(void)
+std::vector<PcmDevice> CoreAudioPlayer::pcm_list()
 {
     UInt32 propsize;
 
@@ -79,7 +90,7 @@ std::vector<PcmDevice> CoreAudioPlayer::pcm_list(void)
         char buf[1024];
         theAddress = {kAudioDevicePropertyDeviceName, kAudioDevicePropertyScopeOutput, 0};
         AudioObjectGetPropertyData(devids[i], &theAddress, 0, NULL, &maxlen, buf);
-        LOG(DEBUG) << "device: " << i << ", name: " << buf << ", channels: " << channels << "\n";
+        LOG(DEBUG, LOG_TAG) << "device: " << i << ", name: " << buf << ", channels: " << channels << "\n";
 
         result.push_back(PcmDevice(i, buf));
     }
@@ -98,21 +109,20 @@ void CoreAudioPlayer::playerCallback(AudioQueueRef queue, AudioQueueBufferRef bu
     size_t bufferedMs = bufferedFrames * 1000 / pubStream_->getFormat().rate() + (ms_ * (NUM_BUFFERS - 1));
     /// 15ms DAC delay. Based on trying.
     bufferedMs += 15;
-    //    LOG(INFO) << "buffered: " << bufferedFrames << ", ms: " << bufferedMs << ", mSampleTime: " << timestamp.mSampleTime << "\n";
+    //    LOG(INFO, LOG_TAG) << "buffered: " << bufferedFrames << ", ms: " << bufferedMs << ", mSampleTime: " << timestamp.mSampleTime << "\n";
 
     /// TODO: sometimes this bufferedMS or AudioTimeStamp wraps around 1s (i.e. we're 1s out of sync (behind)) and recovers later on
     chronos::usec delay(bufferedMs * 1000);
     char* buffer = (char*)bufferRef->mAudioData;
-    if (!pubStream_->getPlayerChunk(buffer, delay, frames_))
+    if (!pubStream_->getPlayerChunkOrSilence(buffer, delay, frames_))
     {
         if (chronos::getTickCount() - lastChunkTick > 5000)
         {
-            LOG(NOTICE) << "No chunk received for 5000ms. Closing Audio Queue.\n";
+            LOG(NOTICE, LOG_TAG) << "No chunk received for 5000ms. Closing Audio Queue.\n";
             uninitAudioQueue(queue);
             return;
         }
-        //		LOG(INFO) << "Failed to get chunk. Playing silence.\n";
-        memset(buffer, 0, buff_size_);
+        // LOG(INFO, LOG_TAG) << "Failed to get chunk. Playing silence.\n";
     }
     else
     {
@@ -130,6 +140,12 @@ void CoreAudioPlayer::playerCallback(AudioQueueRef queue, AudioQueueBufferRef bu
 }
 
 
+bool CoreAudioPlayer::needsThread() const
+{
+    return true;
+}
+
+
 void CoreAudioPlayer::worker()
 {
     while (active_)
@@ -142,7 +158,7 @@ void CoreAudioPlayer::worker()
             }
             catch (const std::exception& e)
             {
-                LOG(ERROR) << "Exception in worker: " << e.what() << "\n";
+                LOG(ERROR, LOG_TAG) << "Exception in worker: " << e.what() << "\n";
                 chronos::sleep(100);
             }
         }
@@ -179,7 +195,7 @@ void CoreAudioPlayer::initAudioQueue()
     frames_ = (sampleFormat.rate() * ms_) / 1000;
     ms_ = frames_ * 1000 / sampleFormat.rate();
     buff_size_ = frames_ * sampleFormat.frameSize();
-    LOG(INFO) << "frames: " << frames_ << ", ms: " << ms_ << ", buffer size: " << buff_size_ << "\n";
+    LOG(INFO, LOG_TAG) << "frames: " << frames_ << ", ms: " << ms_ << ", buffer size: " << buff_size_ << "\n";
 
     AudioQueueBufferRef buffers[NUM_BUFFERS];
     for (int i = 0; i < NUM_BUFFERS; i++)
@@ -189,7 +205,7 @@ void CoreAudioPlayer::initAudioQueue()
         callback(this, queue, buffers[i]);
     }
 
-    LOG(ERROR) << "CoreAudioPlayer::worker\n";
+    LOG(DEBUG, LOG_TAG) << "CoreAudioPlayer::worker\n";
     AudioQueueCreateTimeline(queue, &timeLine_);
     AudioQueueStart(queue, NULL);
     CFRunLoopRun();
@@ -202,3 +218,5 @@ void CoreAudioPlayer::uninitAudioQueue(AudioQueueRef queue)
     pubStream_->clearChunks();
     CFRunLoopStop(CFRunLoopGetCurrent());
 }
+
+} // namespace player
