@@ -17,7 +17,9 @@
 ***/
 
 // local headers
+#include "common/base64.h"
 #include "common/popl.hpp"
+#include "common/utils/string_utils.hpp"
 #include "controller.hpp"
 
 #ifdef HAS_ALSA
@@ -29,6 +31,9 @@
 #ifdef HAS_WASAPI
 #include "player/wasapi_player.hpp"
 #endif
+#ifdef HAS_PIPEWIRE
+#include "player/pipewire_player.hpp"
+#endif
 #include "player/file_player.hpp"
 #ifdef HAS_DAEMON
 #include "common/daemon.hpp"
@@ -37,6 +42,7 @@
 #include "common/aixlog.hpp"
 #include "common/snap_exception.hpp"
 #include "common/str_compat.hpp"
+#include "common/stream_uri.hpp"
 #include "common/version.hpp"
 
 // 3rd party headers
@@ -44,7 +50,10 @@
 #include <boost/asio/signal_set.hpp>
 
 // standard headers
+#include <algorithm>
+#include <filesystem>
 #include <iostream>
+#include <vector>
 #ifndef WINDOWS
 #include <csignal>
 #include <sys/resource.h>
@@ -59,11 +68,12 @@ using namespace std::chrono_literals;
 
 static constexpr auto LOG_TAG = "Snapclient";
 
-
+namespace
+{
 PcmDevice getPcmDevice(const std::string& player, const std::string& parameter, const std::string& soundcard)
 {
     LOG(DEBUG, LOG_TAG) << "Trying to get PCM device for player: " << player << ", parameter: " << ", card: " << soundcard << "\n";
-#if defined(HAS_ALSA) || defined(HAS_PULSE) || defined(HAS_WASAPI)
+#if defined(HAS_ALSA) || defined(HAS_PULSE) || defined(HAS_WASAPI) || defined(HAS_PIPEWIRE)
     vector<PcmDevice> pcm_devices;
 #if defined(HAS_ALSA)
     if (player == player::ALSA)
@@ -76,13 +86,19 @@ PcmDevice getPcmDevice(const std::string& player, const std::string& parameter,
 #if defined(HAS_WASAPI)
     if (player == player::WASAPI)
         pcm_devices = WASAPIPlayer::pcm_list();
+#endif
+#if defined(HAS_PIPEWIRE)
+    if (player == player::PIPEWIRE)
+        pcm_devices = PipeWirePlayer::pcm_list(parameter);
 #endif
     if (player == player::FILE)
         return FilePlayer::pcm_list(parameter).front();
     try
     {
+        // If "soundcard" can be converted to an integer,
+        // try to map the device by index
         int soundcardIdx = cpt::stoi(soundcard);
-        for (auto dev : pcm_devices)
+        for (const auto& dev : pcm_devices)
             if (dev.idx == soundcardIdx)
                 return dev;
     }
@@ -90,7 +106,7 @@ PcmDevice getPcmDevice(const std::string& player, const std::string& parameter,
     {
     }
 
-    for (auto dev : pcm_devices)
+    for (const auto& dev : pcm_devices)
         if (dev.name.find(soundcard) != string::npos)
             return dev;
 #endif
@@ -100,6 +116,7 @@ PcmDevice getPcmDevice(const std::string& player, const std::string& parameter,
     pcm_device.name = soundcard;
     return pcm_device;
 }
+} // namespace
 
 #ifdef WINDOWS
 // hack to avoid case destinction in the signal handler
@@ -127,29 +144,55 @@ int main(int argc, char** argv)
 #ifdef MACOS
 #pragma message "Warning: the macOS support is experimental and might not be maintained"
 #endif
+
+#ifdef HAS_MDNS
+    const std::string default_uri = "tcp://_snapcast._tcp";
+#endif
+
     int exitcode = EXIT_SUCCESS;
     try
     {
         ClientSettings settings;
         string pcm_device(player::DEFAULT_DEVICE);
 
-        OptionParser op("Allowed options");
-        auto helpSwitch = op.add<Switch>("", "help", "produce help message");
-        auto groffSwitch = op.add<Switch, Attribute::hidden>("", "groff", "produce groff message");
-        auto versionSwitch = op.add<Switch>("v", "version", "show version number");
-        op.add<Value<string>>("h", "host", "server hostname or ip address", "", &settings.server.host);
-        op.add<Value<size_t>>("p", "port", "server port", 1704, &settings.server.port);
-        op.add<Value<size_t>>("i", "instance", "instance id when running multiple instances on the same host", 1, &settings.instance);
-        op.add<Value<string>>("", "hostID", "unique host id, default is MAC address", "", &settings.host_id);
+        OptionParser op("Usage: snapclient [options...] [url]\n\n"
+                        " With 'url' = "
+#ifdef HAS_OPENSSL
+                        "<tcp|ws|wss>"
+#else
+                        "<tcp|ws>"
+#endif
+#ifdef HAS_MDNS
+                        "://<snapserver host or IP or mDNS service name>[:port]\n"
+                        " For example: 'tcp://192.168.1.1:1704', or 'ws://)homeserver.local' or 'wss://_snapcast-https._tcp'\n"
+                        " If 'url' is not configured, snapclient defaults to '" +
+                        default_uri + "'\n");
+#else
+                        "://<snapserver host or IP>[:port]\n"
+                        " For example: 'tcp://192.168.1.1:1704', or 'ws://homeserver.local'\n");
+#endif
+        auto helpSwitch = op.add<Switch>("", "help", "Produce help message");
+        auto groffSwitch = op.add<Switch, Attribute::hidden>("", "groff", "Produce groff message");
+        auto versionSwitch = op.add<Switch>("v", "version", "Show version number");
+        auto deprecated_host_opt = op.add<Value<string>>("h", "host", "(deprecated, use [url]) Server hostname or ip address", "");
+        auto deprecated_port_opt = op.add<Value<size_t>>("p", "port", "(deprecated, use [url]) Server port", 1704);
+        op.add<Value<size_t>>("i", "instance", "Instance id when running multiple instances on the same host", 1, &settings.instance);
+        op.add<Value<string>>("", "hostID", "Unique host id, default is MAC address", "", &settings.host_id);
+        op.add<Value<std::filesystem::path>>("", "cert", "Client certificate file (PEM format)", settings.server.certificate, &settings.server.certificate);
+        op.add<Value<std::filesystem::path>>("", "cert-key", "Client private key file (PEM format)", settings.server.certificate_key,
+                                             &settings.server.certificate_key);
+        op.add<Value<string>>("", "key-password", "Key password (for encrypted private key)", settings.server.key_password, &settings.server.key_password);
+        auto server_cert_opt =
+            op.add<Implicit<std::filesystem::path>>("", "server-cert", "Verify server with CA certificate (PEM format)", "default certificates");
 
 // PCM device specific
-#if defined(HAS_ALSA) || defined(HAS_PULSE) || defined(HAS_WASAPI)
-        auto listSwitch = op.add<Switch>("l", "list", "list PCM devices");
-        /*auto soundcardValue =*/op.add<Value<string>>("s", "soundcard", "index or name of the pcm device", pcm_device, &pcm_device);
+#if defined(HAS_ALSA) || defined(HAS_PULSE) || defined(HAS_WASAPI) || defined(HAS_PIPEWIRE)
+        auto listSwitch = op.add<Switch>("l", "list", "List PCM devices");
+        op.add<Value<string>>("s", "soundcard", "Index or name of the PCM device", pcm_device, &pcm_device);
 #endif
-        /*auto latencyValue =*/op.add<Value<int>>("", "latency", "latency of the PCM device", 0, &settings.player.latency);
+        op.add<Value<int>>("", "latency", "Latency of the PCM device", 0, &settings.player.latency);
 #ifdef HAS_SOXR
-        auto sample_format = op.add<Value<string>>("", "sampleformat", "resample audio stream to <rate>:<bits>:<channels>", "");
+        auto sample_format = op.add<Value<string>>("", "sampleformat", "Resample audio stream to <rate>:<bits>:<channels>", "");
 #endif
 
         auto supported_players = Controller::getSupportedPlayerNames();
@@ -160,7 +203,7 @@ int main(int argc, char** argv)
 
 // sharing mode
 #if defined(HAS_OBOE) || defined(HAS_WASAPI)
-        auto sharing_mode = op.add<Value<string>>("", "sharingmode", "audio mode to use [shared|exclusive]", "shared");
+        auto sharing_mode = op.add<Value<string>>("", "sharingmode", "Audio mode to use [shared|exclusive]", "shared");
 #endif
 
         // mixer
@@ -181,14 +224,14 @@ int main(int argc, char** argv)
 // daemon settings
 #ifdef HAS_DAEMON
         int processPriority(-3);
-        auto daemonOption = op.add<Implicit<int>>("d", "daemon", "daemonize, optional process priority [-20..19]", processPriority, &processPriority);
-        auto userValue = op.add<Value<string>>("", "user", "the user[:group] to run snapclient as when daemonized");
+        auto daemonOption = op.add<Implicit<int>>("d", "daemon", "Daemonize, optional process priority [-20..19]", processPriority, &processPriority);
+        auto userValue = op.add<Value<string>>("", "user", "The user[:group] to run snapclient as when daemonized");
 #endif
 
         // logging
-        op.add<Value<string>>("", "logsink", "log sink [null,system,stdout,stderr,file:<filename>]", settings.logging.sink, &settings.logging.sink);
+        op.add<Value<string>>("", "logsink", "Log sink [null,system,stdout,stderr,file:<filename>]", settings.logging.sink, &settings.logging.sink);
         auto logfilterOption = op.add<Value<string>>(
-            "", "logfilter", "log filter <tag>:<level>[,<tag>:<level>]* with tag = * or <log tag> and level = [trace,debug,info,notice,warning,error,fatal]",
+            "", "logfilter", "Log filter <tag>:<level>[,<tag>:<level>]* with tag = * or <log tag> and level = [trace,debug,info,notice,warning,error,fatal]",
             settings.logging.filter);
 
         try
@@ -215,7 +258,7 @@ int main(int argc, char** argv)
 
         settings.player.player_name = utils::string::split_left(settings.player.player_name, ':', settings.player.parameter);
 
-#if defined(HAS_ALSA) || defined(HAS_PULSE) || defined(HAS_WASAPI)
+#if defined(HAS_ALSA) || defined(HAS_PULSE) || defined(HAS_WASAPI) || defined(HAS_PIPEWIRE)
         if (listSwitch->is_set())
         {
             try
@@ -233,6 +276,10 @@ int main(int argc, char** argv)
                 if (settings.player.player_name == player::WASAPI)
                     pcm_devices = WASAPIPlayer::pcm_list();
 #endif
+#if defined(HAS_PIPEWIRE)
+                if (settings.player.player_name == player::PIPEWIRE)
+                    pcm_devices = PipeWirePlayer::pcm_list(settings.player.parameter);
+#endif
 #ifdef WINDOWS
                 // Set console code page to UTF-8 so console known how to interpret string data
                 SetConsoleOutputCP(CP_UTF8);
@@ -303,10 +350,94 @@ int main(int argc, char** argv)
         else
             throw SnapException("Invalid log sink: " + settings.logging.sink);
 
-#if !defined(HAS_AVAHI) && !defined(HAS_BONJOUR)
-        if (settings.server.host.empty())
-            throw SnapException("Snapserver host not configured and mDNS not available, please configure with \"--host\".");
+        if (!op.unknown_options().empty())
+        {
+            throw SnapException("Unknown command line argument: '" + op.unknown_options().front() + "'");
+        }
+
+        if (deprecated_host_opt->is_set() || deprecated_port_opt->is_set())
+        {
+            LOG(WARNING, LOG_TAG) << "Options '--" << deprecated_host_opt->long_name() << "' and '--" << deprecated_port_opt->long_name()
+                                  << "' are deprecated. Please add the server URI as last command line argument\n";
+            settings.server.uri.host = deprecated_host_opt->value();
+            settings.server.uri.port = deprecated_port_opt->value();
+            settings.server.uri.scheme = "tcp";
+        }
+
+        if (!op.non_option_args().empty())
+        {
+            std::vector<std::string> schemes{"tcp", "ws"};
+#ifdef HAS_OPENSSL
+            schemes.emplace_back("wss");
 #endif
+            try
+            {
+                settings.server.uri.parse(op.non_option_args().front());
+            }
+            catch (...)
+            {
+                throw SnapException("Invalid URI - expected format: \"<scheme>://<host or IP>[:port]\", with 'scheme' one of: " +
+                                    utils::string::container_to_string(schemes));
+            }
+
+            if (std::find(schemes.begin(), schemes.end(), settings.server.uri.scheme) == schemes.end())
+                throw SnapException("Protocol must be one of: " + utils::string::container_to_string(schemes));
+
+            if (!settings.server.uri.port.has_value())
+            {
+                if (settings.server.uri.scheme == "tcp")
+                    settings.server.uri.port = 1704;
+                else if (settings.server.uri.scheme == "ws")
+                    settings.server.uri.port = 1780;
+                else if (settings.server.uri.scheme == "wss")
+                    settings.server.uri.port = 1788;
+            }
+
+            if (!settings.server.uri.user.empty() || !settings.server.uri.password.empty())
+            {
+                ClientSettings::Server::Auth auth;
+                auth.scheme = "Basic";
+                auth.param = base64_encode(settings.server.uri.user + ":" + settings.server.uri.password);
+                settings.server.auth = auth;
+            }
+        }
+
+        if (server_cert_opt->is_set())
+        {
+            if (server_cert_opt->get_default() == server_cert_opt->value())
+                settings.server.server_certificate = "";
+            else
+                settings.server.server_certificate = std::filesystem::weakly_canonical(server_cert_opt->value());
+            if (settings.server.server_certificate.value_or("").empty())
+                LOG(INFO, LOG_TAG) << "Server certificate: default certificates\n";
+            else
+                LOG(INFO, LOG_TAG) << "Server certificate: " << settings.server.server_certificate.value_or("") << "\n";
+        }
+
+        if (!settings.server.certificate.empty() && !settings.server.certificate_key.empty())
+        {
+            namespace fs = std::filesystem;
+            settings.server.certificate = fs::weakly_canonical(settings.server.certificate);
+            if (!fs::exists(settings.server.certificate))
+                throw SnapException("Certificate file not found: " + settings.server.certificate.string());
+            settings.server.certificate_key = fs::weakly_canonical(settings.server.certificate_key);
+            if (!fs::exists(settings.server.certificate_key))
+                throw SnapException("Certificate_key file not found: " + settings.server.certificate_key.string());
+        }
+        else if (settings.server.certificate.empty() != settings.server.certificate_key.empty())
+        {
+            throw SnapException("Both SSL 'certificate' and 'certificate_key' must be set or empty");
+        }
+
+        if (settings.server.uri.host.empty())
+        {
+#ifndef HAS_MDNS
+            throw SnapException("Snapserver host not configured and mDNS not available, please configure with \"--host\".");
+#else
+            LOG(INFO, LOG_TAG) << "No server URL given, defaulting to '" << default_uri << "'\n";
+            settings.server.uri.parse(default_uri);
+#endif
+        }
 
 
 #ifdef HAS_DAEMON
@@ -379,6 +510,13 @@ int main(int argc, char** argv)
                      << " \"buffer_time=<total buffer size [ms]>\" - default 80, min 10\n"
                      << " \"fragments=<number of buffers>\" - default 4, min 2\n";
             }
+#endif
+#ifdef HAS_PIPEWIRE
+            else if (settings.player.player_name == player::PIPEWIRE)
+            {
+                cout << "Options are:\n"
+                     << " \"buffer_time=<total buffer size [ms]>\" - default <not set, PipeWire will decide>, min 10\n";
+            }
 #endif
             else
             {
@@ -391,7 +529,10 @@ int main(int argc, char** argv)
 #if defined(HAS_ALSA)
         if (settings.player.pcm_device.idx == -1)
         {
-            LOG(ERROR, LOG_TAG) << "PCM device \"" << pcm_device << "\" not found\n";
+#if defined(HAS_PIPEWIRE)
+            if (settings.player.player_name != player::PIPEWIRE)
+#endif
+                LOG(ERROR, LOG_TAG) << "PCM device \"" << pcm_device << "\" not found\n";
             // exit(EXIT_FAILURE);
         }
 #endif
@@ -445,6 +586,7 @@ int main(int argc, char** argv)
 
         int num_threads = 0;
         std::vector<std::thread> threads;
+        threads.reserve(num_threads);
         for (int n = 0; n < num_threads; ++n)
             threads.emplace_back([&] { io_context.run(); });
         io_context.run();
