@@ -1,6 +1,6 @@
 /***
     This file is part of snapcast
-    Copyright (C) 2014-2024  Johannes Pohl
+    Copyright (C) 2014-2025  Johannes Pohl
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,13 +22,16 @@
 // standard headers
 #include <string>
 
-
+/// Daemonize a process: run a fork as specified user/group
 class Daemon
 {
 public:
-    Daemon(const std::string& user, const std::string& group, const std::string& pidfile);
+    /// c'tor
+    Daemon(std::string user, std::string group, std::string pidfile);
+    /// d'tor
     virtual ~Daemon();
 
+    /// daemonize the process
     void daemonize();
 
 private:
