@@ -66,9 +66,10 @@ enum class message_type : uint16_t
     kHello = 5,
     // kStreamTags = 6,
     kClientInfo = 7,
+    kError = 8,
 
     kFirst = kBase,
-    kLast = kClientInfo
+    kLast = kError
 };
 
 /// Message type to string
@@ -97,6 +98,9 @@ static std::ostream& operator<<(std::ostream& os, const message_type& msg_type)
         case message_type::kClientInfo:
             os << "ClientInfo";
             break;
+        case message_type::kError:
+            os << "Error";
+            break;
         default:
             os << "Unknown";
     }
