@@ -28,6 +28,7 @@
 // standard headers
 #include <algorithm>
 #include <cstdio>
+#include <iomanip>
 #include <map>
 #include <sstream>
 #include <string>
@@ -43,13 +44,13 @@ static constexpr auto LOG_TAG = "StringUtils";
 
 bool wildcardMatch(const std::string& pattern, const std::string& text)
 {
-    LOG(INFO, LOG_TAG) << "wildcardMatch '" << pattern << "', text: '" << text << "'\n";
+    LOG(DEBUG, LOG_TAG) << "wildcardMatch '" << pattern << "', text: '" << text << "'\n";
     std::vector<std::string> parts = utils::string::split(pattern, '*');
     size_t pos = 0;
     for (size_t n = 0; n < parts.size(); ++n)
     {
         const std::string& part = parts[n];
-        LOG(INFO, LOG_TAG) << "Matching '" << part << "', pos: " << pos << "\n";
+        LOG(DEBUG, LOG_TAG) << "Matching '" << part << "', pos: " << pos << "\n";
         pos = text.find(part, pos);
         if (pos == std::string::npos)
             return false;
@@ -106,30 +107,51 @@ std::string trim_copy(const std::string& s)
 }
 
 
-std::string urlEncode(const std::string& str)
+std::string uriEncode(const std::string& str)
 {
-    std::ostringstream os;
-    for (std::string::const_iterator ci = str.begin(); ci != str.end(); ++ci)
+    std::ostringstream escaped;
+    escaped.fill('0');
+    escaped << std::hex;
+
+    for (unsigned char c : str)
     {
-        if ((*ci >= 'a' && *ci <= 'z') || (*ci >= 'A' && *ci <= 'Z') || (*ci >= '0' && *ci <= '9'))
-        { // allowed
-            os << *ci;
-        }
-        else if (*ci == ' ')
+        // Keep alphanumerics and a few safe characters as-is
+        if (isalnum(c) || c == '-' || c == '_' || c == '.' || c == '~')
         {
-            os << '+';
+            escaped << c;
         }
         else
         {
-            auto toHex = [](unsigned char x) { return static_cast<unsigned char>(x + (x > 9 ? ('A' - 10) : '0')); };
-            os << '%' << toHex(*ci >> 4) << toHex(*ci % 16);
+            // Percent-encode all others
+            escaped << '%' << std::uppercase << std::setw(2) << int(c);
+            escaped << std::nouppercase;
         }
     }
 
-    return os.str();
+    return escaped.str();
 }
 
 
+std::string uriEncodePath(const std::string& path)
+{
+    std::stringstream encoded;
+    std::stringstream input(path);
+    std::string segment;
+
+    while (std::getline(input, segment, '/'))
+    {
+        encoded << uriEncode(segment);
+        if (!input.eof())
+        {
+            encoded << "/";
+        }
+    }
+
+    return encoded.str();
+}
+
+
+
 // decode %xx to char
 std::string uriDecode(const std::string& src)
 {
@@ -140,7 +162,7 @@ std::string uriDecode(const std::string& src)
         if (int(src[i]) == 37)
         {
             unsigned int ii;
-            sscanf(src.substr(i + 1, 2).c_str(), "%x", &ii);
+            sscanf(src.substr(i + 1, 2).c_str(), "%x", &ii); // NOLINT
             ch = static_cast<char>(ii);
             ret += ch;
             i += 2;
@@ -170,6 +192,22 @@ void split_left(const std::string& s, char delim, std::string& left, std::string
 }
 
 
+void split_right(const std::string& s, char delim, std::string& left, std::string& right)
+{
+    auto pos = s.rfind(delim);
+    if (pos != std::string::npos)
+    {
+        left = s.substr(0, pos);
+        right = s.substr(pos + 1);
+    }
+    else
+    {
+        left = s;
+        right = "";
+    }
+}
+
+
 std::string split_left(const std::string& s, char delim, std::string& right)
 {
     std::string left;
@@ -178,6 +216,14 @@ std::string split_left(const std::string& s, char delim, std::string& right)
 }
 
 
+std::string split_right(const std::string& s, char delim, std::string& right)
+{
+    std::string left;
+    split_right(s, delim, left, right);
+    return left;
+}
+
+
 
 std::vector<std::string>& split(const std::string& s, char delim, std::vector<std::string>& elems)
 {
