@@ -1,6 +1,6 @@
 /***
     This file is part of snapcast
-    Copyright (C) 2014-2024  Johannes Pohl
+    Copyright (C) 2014-2025  Johannes Pohl
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -48,9 +48,9 @@ void assign(void* pointer, T val)
 } // namespace
 
 
-PcmEncoder::PcmEncoder(const std::string& codecOptions) : Encoder(codecOptions)
+PcmEncoder::PcmEncoder(std::string codecOptions) : Encoder(std::move(codecOptions))
 {
-    headerChunk_.reset(new msg::CodecHeader("pcm"));
+    headerChunk_ = std::make_shared<msg::CodecHeader>("pcm");
 }
 
 
