@@ -1,6 +1,6 @@
 /***
     This file is part of snapcast
-    Copyright (C) 2014-2020  Johannes Pohl
+    Copyright (C) 2014-2025  Johannes Pohl
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,14 +25,17 @@
 namespace encoder
 {
 
+/// PCM encoder
 class PcmEncoder : public Encoder
 {
 public:
-    PcmEncoder(const std::string& codecOptions = "");
+    /// c'tor
+    explicit PcmEncoder(std::string codecOptions);
+
     void encode(const msg::PcmChunk& chunk) override;
     std::string name() const override;
 
-protected:
+private:
     void initEncoder() override;
 };
 
