@@ -3,11 +3,11 @@
      _(  )/ ___) /  \ (  ( \(  _ \(  _ \ / __)( )  ( )
     / \) \\___ \(  O )/    / )   / ) __/( (__(_ _)(_ _)
     \____/(____/ \__/ \_)__)(__\_)(__)   \___)(_)  (_)
-    version 1.3.1
+    version 1.4.1
     https://github.com/badaix/jsonrpcpp
 
     This file is part of jsonrpc++
-    Copyright (C) 2017-2019 Johannes Pohl
+    Copyright (C) 2017-2025 Johannes Pohl
 
     This software may be modified and distributed under the terms
     of the MIT license.  See the LICENSE file for details.
@@ -22,7 +22,10 @@
 #ifndef JSON_RPC_HPP
 #define JSON_RPC_HPP
 
-#include "json.hpp"
+// nlohmann-json
+#include "common/json.hpp"
+
+// standard headers
 #include <cstring>
 #include <exception>
 #include <string>
@@ -151,6 +154,11 @@ public:
         return string_id_;
     }
 
+    bool operator<(const Id& other) const
+    {
+        return (to_json() < other.to_json());
+    }
+
 protected:
     value_t type_;
     int int_id_;
@@ -185,6 +193,8 @@ public:
     bool has(const std::string& key) const;
     bool has(size_t idx) const;
 
+    void add(const std::string& key, const Json& value);
+
     template <typename T>
     T get(const std::string& key) const
     {
@@ -560,7 +570,7 @@ inline void Entity::parse(const char* json_str)
     {
         parse_json(Json::parse(json_str));
     }
-    catch (const RpcException& e)
+    catch (const RpcException&)
     {
         throw;
     }
@@ -686,16 +696,28 @@ inline Parameter::Parameter(const std::string& key1, const Json& value1, const s
 {
     param_map[key1] = value1;
     if (!key2.empty())
-        param_map[key2] = value2;
+        add(key2, value2);
     if (!key3.empty())
-        param_map[key3] = value3;
+        add(key3, value3);
     if (!key4.empty())
-        param_map[key4] = value4;
+        add(key4, value4);
+}
+
+inline void Parameter::add(const std::string& key, const Json& value)
+{
+    param_map[key] = value;
 }
 
 inline void Parameter::parse_json(const Json& json)
 {
-    if (json.is_array())
+    if (json.is_null())
+    {
+        param_array.clear();
+        param_map.clear();
+        type = value_t::null;
+        isNull = true;
+    }
+    else if (json.is_array())
     {
         param_array = json.get<std::vector<Json>>();
         param_map.clear();
@@ -731,7 +753,7 @@ inline bool Parameter::is_map() const
 
 inline bool Parameter::is_null() const
 {
-    return isNull;
+    return type == value_t::null;
 }
 
 inline bool Parameter::has(const std::string& key) const
@@ -743,7 +765,9 @@ inline bool Parameter::has(const std::string& key) const
 
 inline Json Parameter::get(const std::string& key) const
 {
-    return param_map.at(key);
+    if (has(key))
+        return param_map.at(key);
+    throw RpcException("Parameter '" + key + "' not found");
 }
 
 inline bool Parameter::has(size_t idx) const
@@ -790,7 +814,7 @@ inline void Error::parse_json(const Json& json)
         else
             data_ = nullptr;
     }
-    catch (const RpcException& e)
+    catch (const RpcException&)
     {
         throw;
     }
@@ -802,9 +826,7 @@ inline void Error::parse_json(const Json& json)
 
 inline Json Error::to_json() const
 {
-    Json j = {
-        {"code", code_}, {"message", message_},
-    };
+    Json j = {{"code", code_}, {"message", message_}};
 
     if (!data_.is_null())
         j["data"] = data_;
@@ -859,7 +881,7 @@ inline void Request::parse_json(const Json& json)
         else
             params_ = nullptr;
     }
-    catch (const RequestException& e)
+    catch (const RequestException&)
     {
         throw;
     }
@@ -1052,7 +1074,7 @@ inline void Response::parse_json(const Json& json)
         else
             throw RpcException("response must contain result or error");
     }
-    catch (const RpcException& e)
+    catch (const RpcException&)
     {
         throw;
     }
@@ -1064,9 +1086,7 @@ inline void Response::parse_json(const Json& json)
 
 inline Json Response::to_json() const
 {
-    Json j = {
-        {"jsonrpc", "2.0"}, {"id", id_.to_json()},
-    };
+    Json j = {{"jsonrpc", "2.0"}, {"id", id_.to_json()}};
 
     if (error_)
         j["error"] = error_.to_json();
@@ -1116,7 +1136,7 @@ inline void Notification::parse_json(const Json& json)
         else
             params_ = nullptr;
     }
-    catch (const RpcException& e)
+    catch (const RpcException&)
     {
         throw;
     }
@@ -1128,9 +1148,7 @@ inline void Notification::parse_json(const Json& json)
 
 inline Json Notification::to_json() const
 {
-    Json json = {
-        {"jsonrpc", "2.0"}, {"method", method_},
-    };
+    Json json = {{"jsonrpc", "2.0"}, {"method", method_}};
 
     if (params_)
         json["params"] = params_.to_json();
@@ -1240,7 +1258,7 @@ inline entity_ptr Parser::do_parse(const std::string& json_str)
     {
         return do_parse_json(Json::parse(json_str));
     }
-    catch (const RpcException& e)
+    catch (const RpcException&)
     {
         throw;
     }
@@ -1248,8 +1266,10 @@ inline entity_ptr Parser::do_parse(const std::string& json_str)
     {
         throw ParseErrorException(e.what());
     }
-
-    return nullptr;
+    catch (...)
+    {
+        return nullptr;
+    }
 }
 
 inline entity_ptr Parser::do_parse_json(const Json& json)
@@ -1265,7 +1285,7 @@ inline entity_ptr Parser::do_parse_json(const Json& json)
         if (is_batch(json))
             return std::make_shared<Batch>(json);
     }
-    catch (const RpcException& e)
+    catch (const RpcException&)
     {
         throw;
     }
@@ -1283,7 +1303,7 @@ inline bool Parser::is_request(const std::string& json_str)
     {
         return is_request(Json::parse(json_str));
     }
-    catch (const std::exception& e)
+    catch (const std::exception&)
     {
         return false;
     }
@@ -1300,7 +1320,7 @@ inline bool Parser::is_notification(const std::string& json_str)
     {
         return is_notification(Json::parse(json_str));
     }
-    catch (const std::exception& e)
+    catch (const std::exception&)
     {
         return false;
     }
@@ -1317,7 +1337,7 @@ inline bool Parser::is_response(const std::string& json_str)
     {
         return is_response(Json::parse(json_str));
     }
-    catch (const std::exception& e)
+    catch (const std::exception&)
     {
         return false;
     }
@@ -1334,7 +1354,7 @@ inline bool Parser::is_batch(const std::string& json_str)
     {
         return is_batch(Json::parse(json_str));
     }
-    catch (const std::exception& e)
+    catch (const std::exception&)
     {
         return false;
     }
