@@ -1,6 +1,6 @@
 /***
     This file is part of snapcast
-    Copyright (C) 2014-2024  Johannes Pohl
+    Copyright (C) 2014-2025  Johannes Pohl
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,7 +18,6 @@
 
 #pragma once
 
-class PublishBonjour;
 
 // local headers
 #include "publish_mdns.hpp"
@@ -31,12 +30,14 @@ class PublishBonjour;
 #include <thread>
 
 
+/// Bonjour based mDNS publisher
 class PublishBonjour : public PublishmDNS
 {
 public:
-    PublishBonjour(const std::string& serviceName, boost::asio::io_context& ioc);
-    virtual ~PublishBonjour();
-    virtual void publish(const std::vector<mDNSService>& services);
+    /// c'tor
+    PublishBonjour(std::string serviceName, boost::asio::io_context& ioc);
+    ~PublishBonjour() override;
+    void publish(const std::vector<mDNSService>& services) override;
 
 private:
     std::thread pollThread_;
