@@ -1,6 +1,6 @@
 /***
     This file is part of snapcast
-    Copyright (C) 2014-2024  Johannes Pohl
+    Copyright (C) 2014-2025  Johannes Pohl
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -54,7 +54,7 @@ class Server : public StreamMessageReceiver, public ControlMessageReceiver, publ
 
 public:
     /// c'tor
-    Server(boost::asio::io_context& io_context, const ServerSettings& serverSettings);
+    Server(boost::asio::io_context& io_context, ServerSettings serverSettings);
     /// d'tor
     virtual ~Server() = default;
 
@@ -65,7 +65,7 @@ public:
 
 private:
     /// Implementation of StreamMessageReceiver
-    void onMessageReceived(StreamSession* streamSession, const msg::BaseMessage& baseMessage, char* buffer) override;
+    void onMessageReceived(const std::shared_ptr<StreamSession>& streamSession, const msg::BaseMessage& baseMessage, char* buffer) override;
     void onDisconnect(StreamSession* streamSession) override;
 
     /// Implementation of ControllMessageReceiver
