@@ -37,21 +37,29 @@ namespace streamreader
 
 using namespace std::chrono_literals;
 
+/// Boost asio based stream class interface
 template <typename ReadStream>
 class AsioStream : public PcmStream
 {
 public:
-    /// ctor. Encoded PCM data is passed to the PipeListener
-    AsioStream(PcmStream::Listener* pcmListener, boost::asio::io_context& ioc, const ServerSettings& server_settings, const StreamUri& uri);
+    /// c'tor. Encoded PCM data is passed to the PipeListener
+    AsioStream(PcmStream::Listener* pcmListener, boost::asio::io_context& ioc, const ServerSettings& server_settings, const StreamUri& uri,
+               PcmStream::Source source);
 
+    /// Start the stream reader, init the encoder and optionally the stream control
     void start() override;
+    /// Stop the stream reader
     void stop() override;
 
 protected:
+    /// connect to the stream
     virtual void connect() = 0;
+    /// disconnect from the stream
     virtual void disconnect();
 
+    /// resets first chunk timer and starts reader loop
     virtual void on_connect();
+    /// read from asio stream
     virtual void do_read();
     /// Start a timer that will change the stream state to idle after \p duration
     void check_state(const std::chrono::steady_clock::duration& duration);
@@ -62,12 +70,18 @@ protected:
 
     /// Cache last exception to avoid repeated error logging
     std::string lastException_;
-    timeval tv_chunk_;
+    /// Marker: next chunk is the first chunk
     bool first_;
+
+    /// next read = current chunk start + chunk duration
     std::chrono::time_point<std::chrono::steady_clock> nextTick_;
+    /// chunk read buffer
     uint32_t buffer_ms_;
+    /// read timer
     boost::asio::steady_timer read_timer_;
+    /// state timer: set stream to idle/playing on timeout
     boost::asio::steady_timer state_timer_;
+    /// the stream
     std::unique_ptr<ReadStream> stream_;
 
     /// duration of the current silence period
@@ -97,8 +111,9 @@ void AsioStream<ReadStream>::wait(Timer& timer, const std::chrono::duration<Rep,
 
 
 template <typename ReadStream>
-AsioStream<ReadStream>::AsioStream(PcmStream::Listener* pcmListener, boost::asio::io_context& ioc, const ServerSettings& server_settings, const StreamUri& uri)
-    : PcmStream(pcmListener, ioc, server_settings, uri), read_timer_(strand_), state_timer_(strand_)
+AsioStream<ReadStream>::AsioStream(PcmStream::Listener* pcmListener, boost::asio::io_context& ioc, const ServerSettings& server_settings, const StreamUri& uri,
+                                   PcmStream::Source source)
+    : PcmStream(pcmListener, ioc, server_settings, uri, source), read_timer_(strand_), state_timer_(strand_)
 {
     LOG(DEBUG, "AsioStream") << "Chunk duration: " << chunk_->durationMs() << " ms, frames: " << chunk_->getFrameCount() << ", size: " << chunk_->payloadSize
                              << "\n";
