@@ -59,9 +59,13 @@ static constexpr auto SPOTIFY_LOGO = "PD94bWwgdmVyc2lvbj0iMS4wIiBlbmNvZGluZz0iVV
                                      "NzMgMi43Mzl6Ii8+Cjwvc3ZnPgo=";
 
 
-LibrespotStream::LibrespotStream(PcmStream::Listener* pcmListener, boost::asio::io_context& ioc, const ServerSettings& server_settings, const StreamUri& uri)
-    : ProcessStream(pcmListener, ioc, server_settings, uri)
+LibrespotStream::LibrespotStream(PcmStream::Listener* pcmListener, boost::asio::io_context& ioc, const ServerSettings& server_settings, const StreamUri& uri,
+                                 PcmStream::Source source)
+    : ProcessStream(pcmListener, ioc, server_settings, uri, source)
 {
+    if (params_.find("--onevent") != std::string::npos)
+        throw SnapException("please use '&onevent' to configure the '--onevent' script instead of '&param'");
+
     wd_timeout_sec_ = cpt::stoul(uri_.getQuery("wd_timeout", "7800")); ///< 130min
 
     string username = uri_.getQuery("username", "");
@@ -92,7 +96,20 @@ LibrespotStream::LibrespotStream(PcmStream::Listener* pcmListener, boost::asio::
     if (!volume.empty())
         params_ += " --initial-volume " + volume;
     if (!onevent.empty())
-        params_ += " --onevent \"" + onevent + "\"";
+    {
+        std::filesystem::path script = onevent;
+        if (source == Source::rpc)
+        {
+            // script must be located in the [stream] sandbox_dir
+            auto abs_script = utils::file::isInDirectory(script, server_settings.stream.sandbox_dir);
+            if (!abs_script)
+                throw SnapException("onevent script must be located in '" + server_settings.stream.sandbox_dir.native() + "'");
+            script = abs_script.value();
+        }
+        if (!std::filesystem::exists(script))
+            throw SnapException("on event script '" + script.native() + "' does not exist");
+        params_ += " --onevent \"" + script.native() + "\"";
+    }
     if (normalize)
         params_ += " --enable-volume-normalisation";
     if (autoplay)
@@ -103,7 +120,7 @@ LibrespotStream::LibrespotStream(PcmStream::Listener* pcmListener, boost::asio::
         uri_.query["username"] = "xxx";
     if (uri_.query.find("password") != uri_.query.end())
         uri_.query["password"] = "xxx";
-    //	LOG(INFO, LOG_TAG) << "params: " << params << "\n";
+    // LOG(DEBUG, LOG_TAG) << "params: " << params_ << "\n";
 }
 
 
