@@ -29,6 +29,7 @@
 #include "control_error.hpp"
 #include "encoder/encoder_factory.hpp"
 #include "image_cache.hpp"
+#include "streamreader/properties.hpp"
 
 // 3rd party headers
 #include <boost/asio/ip/host_name.hpp>
@@ -45,10 +46,12 @@ namespace streamreader
 static constexpr auto LOG_TAG = "PcmStream";
 
 
-PcmStream::PcmStream(PcmStream::Listener* pcmListener, boost::asio::io_context& ioc, const ServerSettings& server_settings, const StreamUri& uri)
-    : active_(false), strand_(boost::asio::make_strand(ioc.get_executor())), pcmListeners_{pcmListener}, uri_(uri), chunk_ms_(20), state_(ReaderState::kIdle),
-      server_settings_(server_settings), req_id_(0), property_timer_(strand_)
+PcmStream::PcmStream(PcmStream::Listener* pcmListener, boost::asio::io_context& ioc, ServerSettings server_settings, StreamUri uri, Source source)
+    : active_(false), strand_(boost::asio::make_strand(ioc.get_executor())), pcmListeners_{pcmListener}, uri_(std::move(uri)), chunk_ms_(20),
+      state_(ReaderState::kIdle), server_settings_(std::move(server_settings)), req_id_(0), property_timer_(strand_)
 {
+    std::ignore = source;
+
     encoder::EncoderFactory encoderFactory;
     if (uri_.query.find(kUriCodec) == uri_.query.end())
         throw SnapException("Stream URI must have a codec");
@@ -94,7 +97,7 @@ PcmStream::PcmStream(PcmStream::Listener* pcmListener, boost::asio::io_context&
 
 PcmStream::~PcmStream()
 {
-    stop();
+    stop(); // NOLINT
     property_timer_.cancel();
 }
 
@@ -152,7 +155,7 @@ void PcmStream::pollProperties()
             {
                 LOG(INFO, LOG_TAG) << "Response for Plugin.Stream.Player.GetProperties: " << response.to_json() << "\n";
                 if (response.error().code() == 0)
-                    setProperties(response.result());
+                    setProperties(Properties(response.result()));
             });
             pollProperties();
         }
@@ -168,7 +171,7 @@ void PcmStream::onControlNotification(const jsonrpcpp::Notification& notificatio
         if (notification.method() == "Plugin.Stream.Player.Properties")
         {
             LOG(DEBUG, LOG_TAG) << "Received properties notification\n";
-            setProperties(notification.params().to_json());
+            setProperties(Properties(notification.params().to_json()));
         }
         else if (notification.method() == "Plugin.Stream.Ready")
         {
@@ -177,7 +180,7 @@ void PcmStream::onControlNotification(const jsonrpcpp::Notification& notificatio
             {
                 LOG(INFO, LOG_TAG) << "Response for Plugin.Stream.Player.GetProperties: " << response.to_json() << "\n";
                 if (response.error().code() == 0)
-                    setProperties(response.result());
+                    setProperties(Properties(response.result()));
             });
 
             // TODO: Add capabilities or settings?
@@ -228,8 +231,8 @@ void PcmStream::start()
 {
     LOG(DEBUG, LOG_TAG) << "Start: " << name_ << ", type: " << uri_.scheme << ", sampleformat: " << sampleFormat_.toString() << ", codec: " << getCodec()
                         << "\n";
-    encoder_->init([this, self = shared_from_this()](const encoder::Encoder& encoder, std::shared_ptr<msg::PcmChunk> chunk, double duration)
-    { chunkEncoded(encoder, std::move(chunk), duration); }, sampleFormat_);
+    encoder_->init([this, self = shared_from_this()](const encoder::Encoder& encoder, const std::shared_ptr<msg::PcmChunk>& chunk, double duration)
+    { chunkEncoded(encoder, chunk, duration); }, sampleFormat_);
 
     if (stream_ctrl_)
     {
@@ -306,7 +309,7 @@ void PcmStream::setState(ReaderState newState)
 }
 
 
-void PcmStream::chunkEncoded(const encoder::Encoder& encoder, std::shared_ptr<msg::PcmChunk> chunk, double duration)
+void PcmStream::chunkEncoded(const encoder::Encoder& encoder, const std::shared_ptr<msg::PcmChunk>& chunk, double duration)
 {
     std::ignore = encoder;
     // LOG(TRACE, LOG_TAG) << "onChunkEncoded: " << getName() << ", duration: " << duration
@@ -378,7 +381,7 @@ const Properties& PcmStream::getProperties() const
 }
 
 
-void PcmStream::setShuffle(bool shuffle, ResultHandler handler)
+void PcmStream::setShuffle(bool shuffle, ResultHandler&& handler)
 {
     LOG(DEBUG, LOG_TAG) << "setShuffle: " << shuffle << "\n";
     if (!properties_.can_control)
@@ -387,7 +390,7 @@ void PcmStream::setShuffle(bool shuffle, ResultHandler handler)
 }
 
 
-void PcmStream::setLoopStatus(LoopStatus status, ResultHandler handler)
+void PcmStream::setLoopStatus(LoopStatus status, ResultHandler&& handler)
 {
     LOG(DEBUG, LOG_TAG) << "setLoopStatus: " << status << "\n";
     if (!properties_.can_control)
@@ -396,7 +399,7 @@ void PcmStream::setLoopStatus(LoopStatus status, ResultHandler handler)
 }
 
 
-void PcmStream::setVolume(uint16_t volume, ResultHandler handler)
+void PcmStream::setVolume(uint16_t volume, ResultHandler&& handler)
 {
     LOG(DEBUG, LOG_TAG) << "setVolume: " << volume << "\n";
     if (!properties_.can_control)
@@ -405,7 +408,7 @@ void PcmStream::setVolume(uint16_t volume, ResultHandler handler)
 }
 
 
-void PcmStream::setMute(bool mute, ResultHandler handler)
+void PcmStream::setMute(bool mute, ResultHandler&& handler)
 {
     LOG(DEBUG, LOG_TAG) << "setMute: " << mute << "\n";
     if (!properties_.can_control)
@@ -414,7 +417,7 @@ void PcmStream::setMute(bool mute, ResultHandler handler)
 }
 
 
-void PcmStream::setRate(float rate, ResultHandler handler)
+void PcmStream::setRate(float rate, ResultHandler&& handler)
 {
     LOG(DEBUG, LOG_TAG) << "setRate: " << rate << "\n";
     if (!properties_.can_control)
@@ -423,7 +426,7 @@ void PcmStream::setRate(float rate, ResultHandler handler)
 }
 
 
-void PcmStream::setPosition(std::chrono::milliseconds position, ResultHandler handler)
+void PcmStream::setPosition(std::chrono::milliseconds position, ResultHandler&& handler)
 {
     LOG(DEBUG, LOG_TAG) << "setPosition\n";
     if (!properties_.can_seek)
@@ -437,7 +440,7 @@ void PcmStream::setPosition(std::chrono::milliseconds position, ResultHandler ha
 }
 
 
-void PcmStream::seek(std::chrono::milliseconds offset, ResultHandler handler)
+void PcmStream::seek(std::chrono::milliseconds offset, ResultHandler&& handler)
 {
     LOG(DEBUG, LOG_TAG) << "seek\n";
     if (!properties_.can_seek)
@@ -451,7 +454,7 @@ void PcmStream::seek(std::chrono::milliseconds offset, ResultHandler handler)
 }
 
 
-void PcmStream::next(ResultHandler handler)
+void PcmStream::next(ResultHandler&& handler)
 {
     LOG(DEBUG, LOG_TAG) << "next\n";
     if (!properties_.can_go_next)
@@ -460,7 +463,7 @@ void PcmStream::next(ResultHandler handler)
 }
 
 
-void PcmStream::previous(ResultHandler handler)
+void PcmStream::previous(ResultHandler&& handler)
 {
     LOG(DEBUG, LOG_TAG) << "previous\n";
     if (!properties_.can_go_previous)
@@ -469,7 +472,7 @@ void PcmStream::previous(ResultHandler handler)
 }
 
 
-void PcmStream::pause(ResultHandler handler)
+void PcmStream::pause(ResultHandler&& handler)
 {
     LOG(DEBUG, LOG_TAG) << "pause\n";
     if (!properties_.can_pause)
@@ -478,7 +481,7 @@ void PcmStream::pause(ResultHandler handler)
 }
 
 
-void PcmStream::playPause(ResultHandler handler)
+void PcmStream::playPause(ResultHandler&& handler)
 {
     LOG(DEBUG, LOG_TAG) << "playPause\n";
     if (!properties_.can_pause)
@@ -487,7 +490,7 @@ void PcmStream::playPause(ResultHandler handler)
 }
 
 
-void PcmStream::stop(ResultHandler handler)
+void PcmStream::stop(ResultHandler&& handler)
 {
     LOG(DEBUG, LOG_TAG) << "stop\n";
     if (!properties_.can_control)
@@ -496,7 +499,7 @@ void PcmStream::stop(ResultHandler handler)
 }
 
 
-void PcmStream::play(ResultHandler handler)
+void PcmStream::play(ResultHandler&& handler)
 {
     LOG(DEBUG, LOG_TAG) << "play\n";
     if (!properties_.can_play)
@@ -505,13 +508,13 @@ void PcmStream::play(ResultHandler handler)
 }
 
 
-void PcmStream::sendRequest(const std::string& method, const jsonrpcpp::Parameter& params, ResultHandler handler)
+void PcmStream::sendRequest(const std::string& method, const jsonrpcpp::Parameter& params, ResultHandler&& handler)
 {
     if (!stream_ctrl_)
         return handler({ControlErrc::can_not_control});
 
     jsonrpcpp::Request req(++req_id_, method, params);
-    stream_ctrl_->command(req, [handler](const jsonrpcpp::Response& response)
+    stream_ctrl_->command(req, [handler = std::move(handler)](const jsonrpcpp::Response& response)
     {
         if (response.error().code() != 0)
             handler({static_cast<ControlErrc>(response.error().code()), response.error().data()});
