@@ -38,13 +38,23 @@ namespace streamreader
 static constexpr auto LOG_TAG = "ProcessStream";
 
 
-ProcessStream::ProcessStream(PcmStream::Listener* pcmListener, boost::asio::io_context& ioc, const ServerSettings& server_settings, const StreamUri& uri)
-    : AsioStream<stream_descriptor>(pcmListener, ioc, server_settings, uri)
+ProcessStream::ProcessStream(PcmStream::Listener* pcmListener, boost::asio::io_context& ioc, const ServerSettings& server_settings, const StreamUri& uri,
+                             PcmStream::Source source)
+    : AsioStream<stream_descriptor>(pcmListener, ioc, server_settings, uri, source)
 {
     params_ = uri_.getQuery("params");
     wd_timeout_sec_ = cpt::stoul(uri_.getQuery("wd_timeout", "0"));
     LOG(DEBUG, LOG_TAG) << "Watchdog timeout: " << wd_timeout_sec_ << "\n";
     logStderr_ = (uri_.getQuery("log_stderr", "false") == "true");
+
+    if (source == Source::rpc)
+    {
+        auto exe_path = utils::file::isInDirectory(uri.path, server_settings.stream.sandbox_dir);
+        // check if exe is located in sandbox_dir
+        if (!exe_path)
+            throw SnapException("Process stream executable must be located in '" + server_settings.stream.sandbox_dir.native() + "'");
+        uri_.path = exe_path->native();
+    }
 }
 
 
@@ -64,14 +74,14 @@ std::string ProcessStream::findExe(const std::string& filename) const
         return which;
 
     /// check in the same path as this binary
-    char buff[PATH_MAX];
-    char szTmp[32];
-    sprintf(szTmp, "/proc/%d/exe", getpid());
-    ssize_t len = readlink(szTmp, buff, sizeof(buff) - 1);
+    std::array<char, PATH_MAX> buff;
+    std::array<char, 32> szTmp;
+    sprintf(szTmp.data(), "/proc/%d/exe", getpid());
+    ssize_t len = readlink(szTmp.data(), buff.data(), buff.size() - 1);
     if (len != -1)
     {
         buff[len] = '\0';
-        return string(buff) + "/" + exe;
+        return string(buff.data()) + "/" + exe;
     }
 
     return "";
@@ -114,8 +124,8 @@ void ProcessStream::connect()
     on_connect();
     if (wd_timeout_sec_ > 0)
     {
-        watchdog_ = make_unique<Watchdog>(strand_, this);
-        watchdog_->start(std::chrono::seconds(wd_timeout_sec_));
+        watchdog_ = make_unique<Watchdog>(strand_);
+        watchdog_->start(std::chrono::seconds(wd_timeout_sec_), [this](std::chrono::milliseconds ms) { onTimeout(ms); });
     }
     else
     {
@@ -171,7 +181,7 @@ void ProcessStream::stderrReadLine()
 }
 
 
-void ProcessStream::onTimeout(const Watchdog& /*watchdog*/, std::chrono::milliseconds ms)
+void ProcessStream::onTimeout(std::chrono::milliseconds ms)
 {
     LOG(ERROR, LOG_TAG) << "Watchdog timeout: " << ms.count() / 1000 << "s\n";
     if (process_)
