@@ -1,6 +1,6 @@
 /***
     This file is part of snapcast
-    Copyright (C) 2014-2024  Johannes Pohl
+    Copyright (C) 2014-2025  Johannes Pohl
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,6 +27,9 @@
 #ifdef HAS_JACK
 #include "jack_stream.hpp"
 #endif
+#ifdef HAS_PIPEWIRE
+#include "pipewire_stream.hpp"
+#endif
 #include "common/snap_exception.hpp"
 #include "common/str_compat.hpp"
 #include "file_stream.hpp"
@@ -46,21 +49,21 @@ using namespace std;
 namespace streamreader
 {
 
-StreamManager::StreamManager(PcmStream::Listener* pcmListener, boost::asio::io_context& ioc, const ServerSettings& settings)
+StreamManager::StreamManager(PcmStream::Listener* pcmListener, boost::asio::io_context& ioc, ServerSettings settings)
     // const std::string& defaultSampleFormat, const std::string& defaultCodec, size_t defaultChunkBufferMs)
-    : pcmListener_(pcmListener), settings_(settings), io_context_(ioc)
+    : pcmListener_(pcmListener), settings_(std::move(settings)), io_context_(ioc)
 {
 }
 
 
-PcmStreamPtr StreamManager::addStream(const std::string& uri)
+PcmStreamPtr StreamManager::addStream(const std::string& uri, PcmStream::Source source)
 {
     StreamUri streamUri(uri);
-    return addStream(streamUri);
+    return addStream(streamUri, source);
 }
 
 
-PcmStreamPtr StreamManager::addStream(StreamUri& streamUri)
+PcmStreamPtr StreamManager::addStream(StreamUri& streamUri, PcmStream::Source source)
 {
     if (streamUri.query.find(kUriSampleFormat) == streamUri.query.end())
         streamUri.query[kUriSampleFormat] = settings_.stream.sampleFormat;
@@ -89,26 +92,12 @@ PcmStreamPtr StreamManager::addStream(StreamUri& streamUri)
 
     if (streamUri.scheme == "pipe")
     {
-        stream = make_shared<PipeStream>(listener, io_context_, settings_, streamUri);
-    }
-    else if (streamUri.scheme == "file")
-    {
-        stream = make_shared<FileStream>(listener, io_context_, settings_, streamUri);
-    }
-    else if (streamUri.scheme == "process")
-    {
-        stream = make_shared<ProcessStream>(listener, io_context_, settings_, streamUri);
+        stream = make_shared<PipeStream>(listener, io_context_, settings_, streamUri, source);
     }
 #ifdef HAS_ALSA
     else if (streamUri.scheme == "alsa")
     {
-        stream = make_shared<AlsaStream>(listener, io_context_, settings_, streamUri);
-    }
-#endif
-#ifdef HAS_JACK
-    else if (streamUri.scheme == "jack")
-    {
-        stream = make_shared<JackStream>(listener, io_context_, settings_, streamUri);
+        stream = make_shared<AlsaStream>(listener, io_context_, settings_, streamUri, source);
     }
 #endif
     else if ((streamUri.scheme == "spotify") || (streamUri.scheme == "librespot"))
@@ -117,7 +106,7 @@ PcmStreamPtr StreamManager::addStream(StreamUri& streamUri)
         // that all constructors of all parent classes also use the overwritten sample
         // format.
         streamUri.query[kUriSampleFormat] = "44100:16:2";
-        stream = make_shared<LibrespotStream>(listener, io_context_, settings_, streamUri);
+        stream = make_shared<LibrespotStream>(listener, io_context_, settings_, streamUri, source);
     }
     else if (streamUri.scheme == "airplay")
     {
@@ -125,15 +114,35 @@ PcmStreamPtr StreamManager::addStream(StreamUri& streamUri)
         // that all constructors of all parent classes also use the overwritten sample
         // format.
         streamUri.query[kUriSampleFormat] = "44100:16:2";
-        stream = make_shared<AirplayStream>(listener, io_context_, settings_, streamUri);
+        stream = make_shared<AirplayStream>(listener, io_context_, settings_, streamUri, source);
+    }
+    else if (streamUri.scheme == "file")
+    {
+        stream = make_shared<FileStream>(listener, io_context_, settings_, streamUri, source);
+    }
+    else if (streamUri.scheme == "process")
+    {
+        stream = make_shared<ProcessStream>(listener, io_context_, settings_, streamUri, source);
     }
     else if (streamUri.scheme == "tcp")
     {
-        stream = make_shared<TcpStream>(listener, io_context_, settings_, streamUri);
+        stream = make_shared<TcpStream>(listener, io_context_, settings_, streamUri, source);
     }
+#ifdef HAS_PIPEWIRE
+    else if (streamUri.scheme == "pipewire")
+    {
+        stream = make_shared<PipeWireStream>(listener, io_context_, settings_, streamUri, source);
+    }
+#endif
+#ifdef HAS_JACK
+    else if (streamUri.scheme == "jack")
+    {
+        stream = make_shared<JackStream>(listener, io_context_, settings_, streamUri, source);
+    }
+#endif
     else if (streamUri.scheme == "meta")
     {
-        stream = make_shared<MetaStream>(listener, streams_, io_context_, settings_, streamUri);
+        stream = make_shared<MetaStream>(listener, streams_, io_context_, settings_, streamUri, source);
     }
     else
     {
