@@ -23,11 +23,11 @@
 #include "common/aixlog.hpp"
 #include "common/snap_exception.hpp"
 #include "common/str_compat.hpp"
-#include "common/utils/string_utils.hpp"
 
 // 3rd party headers
 
 // standard headers
+#include <cstdint>
 #include <memory>
 
 
@@ -38,17 +38,13 @@ namespace streamreader
 
 static constexpr auto LOG_TAG = "TcpStream";
 
-TcpStream::TcpStream(PcmStream::Listener* pcmListener, boost::asio::io_context& ioc, const ServerSettings& server_settings, const StreamUri& uri)
-    : AsioStream<tcp::socket>(pcmListener, ioc, server_settings, uri), reconnect_timer_(ioc)
+TcpStream::TcpStream(PcmStream::Listener* pcmListener, boost::asio::io_context& ioc, const ServerSettings& server_settings, const StreamUri& uri,
+                     PcmStream::Source source)
+    : AsioStream<tcp::socket>(pcmListener, ioc, server_settings, uri, source), reconnect_timer_(ioc)
 {
+    static constexpr uint16_t DEFAULT_PORT = 4953;
     host_ = uri_.host;
-    auto host_port = utils::string::split(host_, ':');
-    port_ = 4953;
-    if (host_port.size() == 2)
-    {
-        host_ = host_port[0];
-        port_ = cpt::stoi(host_port[1], port_);
-    }
+    port_ = uri_.port.value_or(DEFAULT_PORT);
 
     auto mode = uri_.getQuery("mode", "server");
     if (mode == "server")
