@@ -19,22 +19,24 @@
 // prototype/interface header file
 
 // local headers
-#include "common/aixlog.hpp"
 #include "common/base64.h"
 #include "common/error_code.hpp"
-#include "common/jwt.hpp"
+#include "common/stream_uri.hpp"
+#include "common/utils/file_utils.hpp"
 #include "common/utils/string_utils.hpp"
+// #include "server/jwt.hpp"
 #include "server/authinfo.hpp"
 #include "server/server_settings.hpp"
 #include "server/streamreader/control_error.hpp"
 #include "server/streamreader/properties.hpp"
-#include "server/streamreader/stream_uri.hpp"
 
 // 3rd party headers
 #include <catch2/catch_test_macros.hpp>
 
 // standard headers
-#include <chrono>
+#include <cstddef>
+#include <iostream>
+#include <optional>
 #include <regex>
 #include <system_error>
 #include <vector>
@@ -88,12 +90,44 @@ TEST_CASE("String utils")
 
     std::vector<std::string> vec{"1", "2", "3"};
     REQUIRE(container_to_string(vec) == "1, 2, 3");
+
+    std::string right;
+    std::string left = split_left("left:mid:right", ':', right);
+    REQUIRE(left == "left");
+    REQUIRE(right == "mid:right");
+
+    left = split_right("left:mid:right", ':', right);
+    REQUIRE(left == "left:mid");
+    REQUIRE(right == "right");
+
+    std::string user = split_left("username:password:with:colons:role", ':', right);
+    REQUIRE(user == "username");
+    REQUIRE(right == "password:with:colons:role");
+    std::string role;
+    std::string password = split_right(right, ':', role);
+    REQUIRE(password == "password:with:colons");
+    REQUIRE(role == "role");
 }
 
 
+#ifndef WINDOWS
+TEST_CASE("File utils")
+{
+    using namespace utils::file;
+    REQUIRE(isInDirectory("filename.txt", "/dir/to/check").value() == "/dir/to/check/filename.txt");
+    REQUIRE(isInDirectory("/dir/to/check/filename.txt", "/dir/to/check") == "/dir/to/check/filename.txt");
+    REQUIRE(isInDirectory("/dir/to/check/subdir/filename.txt", "/dir/to/check") == "/dir/to/check/subdir/filename.txt");
+    REQUIRE(isInDirectory("/dir/to/check/subdir1/../filename.txt", "/dir/to/check") == "/dir/to/check/filename.txt");
+    REQUIRE(isInDirectory("/dir/to/check_xxx/filename.txt", "/dir/to/check") == std::nullopt);
+    REQUIRE(isInDirectory("/dir/to/check/subdir1/../../filename.txt", "/dir/to/check") == std::nullopt);
+    REQUIRE(isInDirectory("/dir/xx/filename.txt", "/dir/to/check") == std::nullopt);
+}
+#endif
+
+
+#if 0
 TEST_CASE("JWT")
 {
-#if 0
     /// ECDSA
     {
         const auto* key = "-----BEGIN EC PRIVATE KEY-----\n"
@@ -137,7 +171,6 @@ TEST_CASE("JWT")
         REQUIRE(jwt.getIat().value() == std::chrono::seconds(1516239022));
         REQUIRE(!jwt.getExp().has_value());
     }
-#endif
 
     /// RSA keys
     {
@@ -220,21 +253,39 @@ TEST_CASE("JWT")
         REQUIRE(!jwt.getExp().has_value());
     }
 }
+#endif
 
 
 TEST_CASE("Uri")
 {
-    using namespace streamreader;
     StreamUri uri("pipe:///tmp/snapfifo?name=default&codec=flac");
     REQUIRE(uri.scheme == "pipe");
     REQUIRE(uri.path == "/tmp/snapfifo");
     REQUIRE(uri.host.empty());
 
+    uri = StreamUri("wss://user:pass@localhost:1788");
+    REQUIRE(uri.scheme == "wss");
+    REQUIRE(uri.path.empty());
+    REQUIRE(uri.host == "localhost");
+    REQUIRE(uri.user == "user");
+    REQUIRE(uri.password == "pass");
+    REQUIRE(uri.port == 1788);
+
+    uri = StreamUri("wss://user@localhost:1788");
+    REQUIRE(uri.scheme == "wss");
+    REQUIRE(uri.path.empty());
+    REQUIRE(uri.host == "localhost");
+    REQUIRE(uri.user == "user");
+    REQUIRE(uri.password.empty());
+    REQUIRE(uri.port == 1788);
+
     // uri = StreamUri("scheme:[//host[:port]][/]path[?query=none][#fragment]");
     // Test with all fields
-    uri = StreamUri("scheme://host:port/path?query=none&key=value#fragment");
+    uri = StreamUri("scheme://host:42/path?query=none&key=value#fragment");
     REQUIRE(uri.scheme == "scheme");
-    REQUIRE(uri.host == "host:port");
+    REQUIRE(uri.host == "host");
+    REQUIRE(uri.port.has_value());
+    REQUIRE(uri.port.value() == 42);
     REQUIRE(uri.path == "/path");
     REQUIRE(uri.query["query"] == "none");
     REQUIRE(uri.query["key"] == "value");
@@ -243,13 +294,16 @@ TEST_CASE("Uri")
     // Test with all fields, url encoded
     // "%21%23%24%25%26%27%28%29%2A%2B%2C%2F%3A%3B%3D%3F%40%5B%5D"
     // "!#$%&'()*+,/:;=?@[]"
-    uri = StreamUri("scheme%26://%26host%3f:port/pa%2Bth?%21%23%24%25%26%27%28%29=%2A%2B%2C%2F%3A%3B%3D%3F%40%5B%5D&key%2525=value#fragment%3f%21%3F");
-    REQUIRE(uri.scheme == "scheme&");
-    REQUIRE(uri.host == "&host?:port");
+    uri = StreamUri("scheme://host:23/pa%2Bth?%21%23%24%25%26%27%28%29=%2A%2B%2C%2F%3A%3B%3D%3F%40%5B%5D&key%2525=value#fragment%3F%21%3F");
+    REQUIRE(uri.scheme == "scheme");
+    REQUIRE(uri.host == "host");
+    REQUIRE(uri.port.has_value());
+    REQUIRE(uri.port.value() == 23);
     REQUIRE(uri.path == "/pa+th");
     REQUIRE(uri.query["!#$%&'()"] == "*+,/:;=?@[]");
     REQUIRE(uri.query["key%25"] == "value");
     REQUIRE(uri.fragment == "fragment?!?");
+    REQUIRE(uri.toString() == uri.uri);
 
     // No host
     uri = StreamUri("scheme:///path?query=none#fragment");
@@ -283,6 +337,7 @@ TEST_CASE("Uri")
     uri = StreamUri("spotify:///librespot?name=Spotify&username=EMAIL&password=string%26with%26ampersands&devicename=Snapcast&bitrate=320&killall=false");
     REQUIRE(uri.scheme == "spotify");
     REQUIRE(uri.host.empty());
+    REQUIRE(!uri.port.has_value());
     REQUIRE(uri.path == "/librespot");
     REQUIRE(uri.query["name"] == "Spotify");
     REQUIRE(uri.query["username"] == "EMAIL");
@@ -292,7 +347,20 @@ TEST_CASE("Uri")
     REQUIRE(uri.query["killall"] == "false");
     REQUIRE(uri.toString().find("spotify:///librespot?") == 0);
     StreamUri uri_from_str{uri.toString()};
-    // REQUIRE(uri == uri_from_str);
+    REQUIRE(uri == uri_from_str);
+
+    // Issue #1410
+    uri = StreamUri("tcp://0.0.0.0:5099?sampleformat=48000:16:2&idle_threshold=60000&name=Music Assistant");
+    REQUIRE(
+        uri.toJson().dump() ==
+        R"({"fragment":"","host":"0.0.0.0:5099","path":"","query":{"idle_threshold":"60000","name":"Music Assistant","sampleformat":"48000:16:2"},"raw":"tcp://0.0.0.0:5099?idle_threshold=60000&name=Music%20Assistant&sampleformat=48000%3A16%3A2","scheme":"tcp"})");
+    REQUIRE(uri.toJson()["host"].get<std::string>() == "0.0.0.0:5099");
+
+    uri = StreamUri("tcp://0.0.0.0?sampleformat=48000:16:2&idle_threshold=60000&name=Music Assistant");
+    REQUIRE(
+        uri.toJson().dump() ==
+        R"({"fragment":"","host":"0.0.0.0","path":"","query":{"idle_threshold":"60000","name":"Music Assistant","sampleformat":"48000:16:2"},"raw":"tcp://0.0.0.0?idle_threshold=60000&name=Music%20Assistant&sampleformat=48000%3A16%3A2","scheme":"tcp"})");
+    REQUIRE(uri.toJson()["host"].get<std::string>() == "0.0.0.0");
 }
 
 
@@ -457,8 +525,7 @@ TEST_CASE("Librespot2")
     }
 
     line = "[2021-06-04T07:20:47Z INFO  librespot_playback::player] metadata:{\"ARTIST\":\"artist\",\"TITLE\":\"title\"}";
-    n = 0;
-    if (((n = line.find("metadata:")) != std::string::npos))
+    if (n = line.find("metadata:"); n != std::string::npos)
     {
         std::string meta = line.substr(n + 9);
         REQUIRE(meta == "{\"ARTIST\":\"artist\",\"TITLE\":\"title\"}");
@@ -642,7 +709,7 @@ TEST_CASE("ErrorOr")
         // Move value out
         REQUIRE(error_or.takeValue() == "test");
         // Value has been moved out, get will return an empty string
-        REQUIRE(error_or.getValue().empty());
+        // REQUIRE(error_or.getValue().empty());
     }
 
     {
@@ -687,47 +754,89 @@ TEST_CASE("WildcardMatch")
 
 TEST_CASE("Auth")
 {
+    ServerSettings::Authorization auth_settings;
+    auth_settings.enabled = true;
     {
-        ServerSettings settings;
-        ServerSettings::User user("badaix:*:secret");
-        REQUIRE(user.permissions.size() == 1);
-        REQUIRE(user.permissions[0] == "*");
-        settings.users.push_back(user);
+        auth_settings.init({"admin:*"}, {"badaix:secret:admin"});
+        REQUIRE(auth_settings.users.size() == 1);
+        REQUIRE(auth_settings.roles.size() == 1);
+        REQUIRE(auth_settings.users.front().role->role == "admin");
+        REQUIRE(auth_settings.users.front().role->permissions.size() == 1);
+        REQUIRE(auth_settings.users.front().role->permissions.front() == "*");
 
-        AuthInfo auth(settings);
+        AuthInfo auth(auth_settings);
         auto ec = auth.authenticateBasic(base64_encode("badaix:secret"));
         REQUIRE(!ec);
-        REQUIRE(auth.hasAuthInfo());
+        REQUIRE(auth.isAuthenticated());
         REQUIRE(auth.hasPermission("stream"));
     }
 
     {
-        ServerSettings settings;
-        ServerSettings::User user("badaix::secret");
-        REQUIRE(user.permissions.empty());
-        settings.users.push_back(user);
+        auth_settings.init({"admin:"}, {"badaix:secret:admin"});
+        REQUIRE(auth_settings.users.size() == 1);
+        REQUIRE(auth_settings.roles.size() == 1);
+        REQUIRE(auth_settings.users.front().role->role == "admin");
+        REQUIRE(auth_settings.users.front().role->permissions.empty());
 
-        AuthInfo auth(settings);
+        AuthInfo auth(auth_settings);
         auto ec = auth.authenticateBasic(base64_encode("badaix:secret"));
         REQUIRE(!ec);
-        REQUIRE(auth.hasAuthInfo());
+        REQUIRE(auth.isAuthenticated());
         REQUIRE(!auth.hasPermission("stream"));
     }
 
     {
-        ServerSettings settings;
-        ServerSettings::User user("badaix:*:secret");
-        settings.users.push_back(user);
+        auth_settings.init({}, {"badaix:secret:"});
+        REQUIRE(auth_settings.users.size() == 1);
+        REQUIRE(auth_settings.roles.empty());
+        REQUIRE(auth_settings.users.front().role->permissions.empty());
 
-        AuthInfo auth(settings);
+        AuthInfo auth(auth_settings);
+        auto ec = auth.authenticateBasic(base64_encode("badaix:secret"));
+        REQUIRE(!ec);
+        REQUIRE(auth.isAuthenticated());
+        REQUIRE(!auth.hasPermission("stream"));
+    }
+
+    {
+        auth_settings.init({"admin:xxx,stream"}, {"badaix:secret:admin"});
+        REQUIRE(auth_settings.users.size() == 1);
+        REQUIRE(auth_settings.roles.size() == 1);
+        REQUIRE(auth_settings.users.front().role->permissions.size() == 2);
+        REQUIRE(auth_settings.users.front().role->permissions[0] == "xxx");
+        REQUIRE(auth_settings.users.front().role->permissions[1] == "stream");
+
+        AuthInfo auth(auth_settings);
         auto ec = auth.authenticateBasic(base64_encode("badaix:wrong_password"));
         REQUIRE(ec == AuthErrc::wrong_password);
-        REQUIRE(!auth.hasAuthInfo());
+        REQUIRE(!auth.isAuthenticated());
         REQUIRE(!auth.hasPermission("stream"));
 
         ec = auth.authenticateBasic(base64_encode("unknown_user:secret"));
         REQUIRE(ec == AuthErrc::unknown_user);
-        REQUIRE(!auth.hasAuthInfo());
+        REQUIRE(!auth.isAuthenticated());
         REQUIRE(!auth.hasPermission("stream"));
+
+        ec = auth.authenticateBasic(base64_encode("badaix:secret"));
+        REQUIRE(!ec);
+        REQUIRE(auth.isAuthenticated());
+        REQUIRE(auth.hasPermission("stream"));
+        REQUIRE(!auth.hasPermission("play"));
+    }
+
+    {
+        auth_settings.init({"admin:Client.*,Group.*,-Group.Set*"}, {"badaix:secret:admin"});
+        REQUIRE(auth_settings.users.size() == 1);
+        REQUIRE(auth_settings.roles.size() == 1);
+        REQUIRE(auth_settings.users.front().role->permissions.size() == 3);
+
+        AuthInfo auth(auth_settings);
+        auto ec = auth.authenticateBasic(base64_encode("badaix:secret"));
+        REQUIRE(!ec);
+        REQUIRE(auth.hasPermission("Client.SetVolume"));
+        REQUIRE(auth.hasPermission("Client.SetName"));
+        REQUIRE(auth.hasPermission("Group.GetStatus"));
+        REQUIRE(!auth.hasPermission("Group.SetName"));
+        REQUIRE(!auth.hasPermission("Server.GetStatus"));
     }
 }
